﻿using CalendarNotepad.Extends;
using CalendarNotepad.Models;
using SqlSugar;

namespace CalendarNotepad
{
    public partial class WorkMain:Form
    {
        public DbContext Db { get; set; } = new DbContext();

        public WorkMain() {
            InitializeComponent();
            this.rtbMsg.EnableAutoDragDrop = true;
            this.DataReset();
            this.rtbMsg.TextChanged += RtbMsg_TextChanged;
            foreach(var c in this.C_WorkTypes.Controls) {
                var rab = c as RadioButton;
                if(rab != null) {
                    rab.CheckedChanged += Rab_CheckedChanged;
                }
            }
        }

        private void Rab_CheckedChanged(object? sender,EventArgs e) {
            this.DataChanged();
        }

        private void RtbMsg_TextChanged(object? sender,EventArgs e) {
            this.DataChanged();
        }

        private void WorkMain_FormClosing(object sender,FormClosingEventArgs e) {
            if(this.Db != null) {
                this.Db.Close();
                this.Db.Dispose();
            }
        }

        private void mcWorkDay_DateChanged(object sender,DateRangeEventArgs e) {
            var wd = this.mcWorkDay.SelectionStart.ToString("yyyyMMdd");
            var sr = "全天";
            var qu = this.Db.Queryable<WorkUnit>().Where(a => a.DayRange == sr && a.WorkDay == wd).ToList();
            if(qu.Any()) {
                var fir = qu.First();
                this.rtbMsg.Text = fir.WorkMessage;
                this.SetWorkType(fir.WorkType);
            }
            else {
                this.rtbMsg.Text = "";
                this.SetWorkType("休假");
            }
            this.DataReset();
            Task.Factory.StartNew(() => {
                this.Invoke(() => {
                    this.mcWorkDay.BoldedDates = GetBoldDays().ToArray();
                });
            });
        }

        private List<DateTime> GetBoldDays() {
            var year = this.mcWorkDay.SelectionStart.Year;
            var month = this.mcWorkDay.SelectionStart.Month;
            var days = new List<DateTime>();
            for(int i = 1;i < 32;i++) {
                var wd = $"{year}{month.ToString("D2")}{i.ToString("D2")}";
                var qu = this.Db.Queryable<WorkUnit>().Where(a => a.WorkDay == wd && a.WorkMessage != "").ToList();
                if(qu.Any()) {
                    days.Add(new DateTime(year,month,i));
                }
            }

            return days;
        }

        private void WorkMain_Load(object sender,EventArgs e) {
            mcWorkDay_DateChanged(null,null);
        }

        /// <summary>
        /// 获取工作类型
        /// </summary>
        /// <returns></returns>
        private string GetWorkType() {
            foreach(Control c in this.C_WorkTypes.Controls) {
                var rb = c as RadioButton;
                if(rb != null && rb.Checked) {
                    return rb.Text;
                }
            }
            return "";
        }
        /// <summary>
        /// 设置工作类型
        /// </summary>
        /// <param name="t">工作类型</param>
        private void SetWorkType(string? t) {
            if(t.IsNullOrEmpty()) {
                return;
            }
            foreach(Control c in this.C_WorkTypes.Controls) {
                var rb = c as RadioButton;
                if(rb != null && rb.Text == t) {
                    rb.Checked = true;
                    return;
                }
            }
        }

        private void btSave_Click(object sender,EventArgs e) {
            var wd = this.mcWorkDay.SelectionStart.ToString("yyyyMMdd");
            var dr = "全天";
            var msg = this.rtbMsg.Text;
            var wt = GetWorkType();
            var exp = Expressionable.Create<WorkUnit>().And(a => a.DayRange == dr && a.WorkDay == wd).ToExpression();
            var qu = this.Db.Queryable<WorkUnit>().Where(exp).ToList();
            if(qu.Any() && msg.IsNullOrEmpty()) {
                this.Db.Deleteable<WorkUnit>().Where(exp).ExecuteCommand();
                this.DataReset();
                return;
            }
            if(qu.Any() && msg.IsNotNullOrEmpty()) {
                this.Db.Updateable<WorkUnit>().Where(exp)
                    .SetColumns(a => a.WorkMessage == msg)
                    .SetColumns(a => a.WorkType == wt)
                    .ExecuteCommand();
                this.DataReset();
                return;
            }
            if(!qu.Any() && msg.IsNotNullOrEmpty()) {
                this.Db.Insertable<WorkUnit>(new WorkUnit {
                    DayRange = dr,WorkDay = wd,WorkMessage = msg,WorkType = wt,
                }).ExecuteCommand();
                this.DataReset();
                return;
            }
        }

        private void btReset_Click(object sender,EventArgs e) {
            mcWorkDay_DateChanged(sender,null);
            this.DataReset();
        }

        private void DataChanged() {
            this.btSave.Enabled = true;
            this.btReset.Enabled = true;
        }
        private void DataReset() {
            this.btSave.Enabled = false;
            this.btReset.Enabled = false;
        }
    }
}
